import { onBeforeUnmount, onMounted } from 'vue';
import { Editor, EditorEvents } from '../editor';

export const useEditorEvents = (editor: Editor) => {
  const handlers: {
    [x: string]: (...args: any) => void;
  } = {};

  const register = <E extends keyof EditorEvents>(
    event: E,
    fn: EditorEvents[E]
  ) => {
    handlers[event as string] = fn;
  };

  onMounted(() => {
    Object.keys(handlers).forEach((event) => {
      editor.events.addListener(event as keyof EditorEvents, handlers[event]);
    });
  });

  onBeforeUnmount(() => {
    Object.keys(handlers).forEach((event) => {
      editor.events.removeEventListener(
        event as keyof EditorEvents,
        handlers[event]
      );
    });
  });

  return { register };
};
