import { EngineComponent, EventEmitter, Renderer } from '@freeblox/engine';
import { EditorEvents } from '../types/events';

/**
 * Provides editing shortcuts for the editor.
 */
export class ShortcutsComponent extends EngineComponent {
  public name = ShortcutsComponent.name;
  public cleanUpEvents?: Function;

  constructor(
    protected renderer: Renderer,
    protected events: EventEmitter<EditorEvents>
  ) {
    super(renderer, events);
  }

  initialize(): void {
    this.cleanUpEvents = this.bindEvents();
  }

  update(delta: number): void {}

  dispose(): void {
    this.cleanUpEvents?.call(this);
  }

  private bindEvents() {
    const keyPressEvent = (event: KeyboardEvent) => {
      if (
        !event.target ||
        ((event.target as HTMLElement).tagName !== 'BODY' &&
          !(event.target as HTMLElement).hasAttribute('data-editor-events'))
      )
        return;

      if (event.key === 'Delete') {
        this.events.emit('delete');
        return;
      }

      if (!event.ctrlKey) return;
      event.preventDefault();
      switch (event.key) {
        case 'c':
          this.events.emit('copy');
          break;
        case 'v':
          this.events.emit('paste', undefined);
          break;
        case 'x':
          this.events.emit('cut');
          break;
        case 'z':
          this.events.emit('undo');
          break;
        case 'd':
          this.events.emit('duplicate');
          break;
        case 'y':
          this.events.emit('redo');
          break;
        case 'g':
          this.events.emit('group');
          break;
        case 'a':
          this.events.emit('selectAll');
          break;
      }
    };

    window.addEventListener('keydown', keyPressEvent);

    return () => {
      window.removeEventListener('keydown', keyPressEvent);
    };
  }
}
