import {
  PerspectiveCamera,
  Scene,
  Vector2,
  WebGLRenderer as ThreeWebGLRenderer,
  WebGLRendererParameters,
  ColorRepresentation,
  Color,
} from 'three';
import { Renderer } from '../types/renderer';

export class WebGLRenderer implements Renderer {
  public renderer = new ThreeWebGLRenderer(this.params);
  public camera = new PerspectiveCamera(
    75,
    this.resolution.x / this.resolution.y,
    0.1,
    10000
  );
  public scene = new Scene();

  constructor(
    public viewport: HTMLElement,
    public resolution = new Vector2(1080, 720),
    private params?: WebGLRendererParameters
  ) {
    this.renderer.setSize(resolution.x, resolution.y);
    viewport.appendChild(this.renderer.domElement);
    this.renderer.autoClear = false;
  }

  reset(params?: WebGLRendererParameters) {
    const color = new Color();
    const alpha = this.renderer.getClearAlpha();
    this.renderer.getClearColor(color);
    this.viewport.removeChild(this.renderer.domElement);
    this.renderer.dispose();

    this.renderer = new ThreeWebGLRenderer({ ...this.params, ...params });
    this.renderer.setSize(this.resolution.x, this.resolution.y);
    this.renderer.autoClear = false;
    this.renderer.setClearColor(color, alpha);
    this.viewport.appendChild(this.renderer.domElement);
  }

  setSize(width: number, height: number) {
    this.resolution.set(width, height);
    this.camera.aspect = this.resolution.x / this.resolution.y;
    this.renderer.setSize(this.resolution.x, this.resolution.y);
    this.camera.updateProjectionMatrix();
  }

  render() {
    this.renderer.clear();
    this.renderer.render(this.scene, this.camera);
  }

  dispose() {
    this.renderer.dispose();
  }

  setClearColor(color: ColorRepresentation, alpha?: number): void {
    this.renderer.setClearColor(color, alpha);
  }
}
