import {
  ColorRepresentation,
  PerspectiveCamera,
  Scene,
  Renderer as ThreeRenderer,
  Vector2,
} from 'three';

export interface Renderer {
  /**
   * Three.js renderer.
   */
  renderer: ThreeRenderer;

  /**
   * Three.js camera.
   */
  camera: PerspectiveCamera;

  /**
   * Three.js scene.
   */
  scene: Scene;

  /**
   * Viewport element.
   */
  viewport: HTMLElement;

  /**
   * Resolution.
   */
  resolution: Vector2;

  /**
   * Set new size.
   * @param width Width
   * @param height Height
   */
  setSize(width: number, height: number): void;

  /**
   * Render a frame.
   */
  render(): void;

  /**
   * Dispose of the renderer.
   */
  dispose(): void;

  /**
   * Set renderer clear (background) color.
   * @param color New color
   * @param alpha Alpha
   */
  setClearColor(color: ColorRepresentation, alpha?: number): void;

  /**
   * Reset current renderer. Use new properties, if applicable.
   * @param args Renderer arguments
   */
  reset(...args: any[]): void;
}
